@@ -23,16 +23,23 @@ class SlashClient:
     Parameters
     ----------
     client : :class:`commands.Bot` | :class:`commands.AutoShardedBot`
+        The discord.py Bot instance
+    show_warnings : :class:`bool`
+        Whether to show the warnings or not
+    modify_send : :class:`bool`
+        Whether to modify :class:`Messageable.send` and :class:`Message.edit`
 
     Attributes
     ----------
     client : :class:`commands.Bot` | :class:`commands.AutoShardedBot`
     global_commands : :class:`list`
         All registered global commands are cached here
+    commands : :class:`list`
+        All working slash commands
     is_ready : bool
         Equals to ``True`` if SlashClient is ready, otherwise it's ``False``
     '''
-    def __init__(self, client, *, show_warnings: bool=False):
+    def __init__(self, client, *, show_warnings: bool=False, modify_send: bool=True):
         _HANDLER.client = client
         self.client = _HANDLER.client
         self.events = {}
@@ -40,19 +47,29 @@ class SlashClient:
         self._global_commands = {}
         self._guild_commands = {}
         self._show_warnings = show_warnings
+        self._modify_send = modify_send
         self.active_shard_count = 0
         self.is_ready = False
-        self.client.add_listener(self._on_shard_connect, 'on_shard_connect')
-        self.client.add_listener(self._on_connect, 'on_connect')
-        self.client.add_listener(self._on_guild_remove, 'on_guild_remove')
+        # Add listeners
+        self._register_listeners()
         # Modify old discord.py methods
         self._modify_discord()
-        # Link the slash ext to client
-        self.client.slash = self
+        # Link the slash ext to client if doesn't already exists
+        if not hasattr(self.client, "slash"):
+            self.client.slash = self
         # Inject cogs that are already loaded
         for cog in self.client.cogs.values():
             self._inject_cogs(cog)
     
+    def _register_listeners(self):
+        self.client.add_listener(self._on_guild_remove, 'on_guild_remove')
+        self.client.add_listener(self._on_socket_response, 'on_socket_response')
+        if isinstance(self.client, discord.AutoShardedClient):
+            self.client.add_listener(self._on_shard_connect, 'on_shard_connect')
+            self.client.add_listener(self._on_ready, 'on_ready')
+        else:
+            self.client.add_listener(self._on_connect, 'on_connect')
+
     def _modify_discord(self):
         # Modify cog loader
         _add_cog = self.client.add_cog
@@ -72,12 +89,77 @@ class SlashClient:
                 messageable = messageable.channel
             return await self._send_with_components(messageable, *args, **kwargs)
         
+        async def edit_with_components(message, *args, **kwargs):
+            return await self._edit_with_components(message, *args, **kwargs)
+        
         async def ctx_wait_for_button_click(ctx, check=None, timeout=None):
             return await self.wait_for_button_click(check=check, timeout=timeout)
 
-        Messageable.send = send_with_components
-        discord.Message.edit = self._edit_with_components
+        async def fetch_commands(guild):
+            return await self.fetch_guild_commands(guild.id)
+        
+        async def fetch_command(guild, command_id):
+            return await self.fetch_guild_command(guild.id, command_id)
+        
+        async def edit_command(guild, command_id, slash_command):
+            return await self.edit_guild_slash_command(guild.id, command_id, slash_command)
+        
+        async def edit_command_permissions(guild, command_id, permissions):
+            return await self.edit_guild_command_permissions(guild.id, command_id, permissions)
+
+        async def batch_edit_command_permissions(guild, permissions):
+            return await self.batch_edit_guild_command_permissions(guild.id, permissions)
+
+        async def delete_command(guild, command_id):
+            return await self.delete_guild_command(guild.id, command_id)
+        
+        async def delete_commands(guild):
+            return await self.delete_guild_commands(guild.id)
+
+        def get_commands(guild):
+            return self.get_guild_commands(guild.id)
+        
+        def get_command(guild, command_id):
+            return self.get_guild_command(guild.id, command_id)
+        
+        def get_command_named(guild, name):
+            return self.get_guild_command_named(guild.id, name)
+
+        if self._modify_send:
+            Messageable.send = send_with_components
+            discord.Message.edit = edit_with_components
         Context.wait_for_button_click = ctx_wait_for_button_click
+        discord.Guild.get_commands = get_commands
+        discord.Guild.get_command = get_command
+        discord.Guild.get_command_named = get_command_named
+        discord.Guild.fetch_commands = fetch_commands
+        discord.Guild.fetch_command = fetch_command
+        discord.Guild.edit_command = edit_command
+        discord.Guild.edit_command_permissions = edit_command_permissions
+        discord.Guild.batch_edit_command_permissions = batch_edit_command_permissions
+        discord.Guild.delete_command = delete_command
+        discord.Guild.delete_commands = delete_commands
+
+    def teardown(self):
+        '''Cleanup the client by removing all registered listeners and caches.'''
+        self.client.remove_listener(self._on_guild_remove, 'on_guild_remove')
+        self.client.remove_listener(self._on_socket_response, 'on_socket_response')
+        if isinstance(self.client, discord.AutoShardedClient):
+            self.client.remove_listener(self._on_shard_connect, 'on_shard_connect')
+            self.client.remove_listener(self._on_ready, 'on_ready')
+        else:
+            self.client.remove_listener(self._on_connect, 'on_connect')
+
+        self.events.clear()
+        self._listeners.clear()
+        self._global_commands.clear()
+        self._guild_commands.clear()
+        if hasattr(self.client, "slash"):
+            del self.client.slash
+        self.is_ready = False
+
+    def __del__(self):
+        self.teardown()
 
     @property
     def commands(self):
@@ -797,11 +879,15 @@ class SlashClient:
     def _eject_cogs(self, name):
         _HANDLER.commands = {kw: cmd for kw, cmd in _HANDLER.commands.items() if cmd._cog_name != name}
 
-    def _do_invokation(self, payload):
-        '''
-        # Don't use it
-        '''
-        self.client.loop.create_task(self._invoke_slash_command(payload))
+    def _guilds_with_commands(self):
+        guilds = set()
+        for cmd in _HANDLER.commands.values():
+            if cmd.guild_ids is not None:
+                guilds = guilds.union(set(cmd.guild_ids))
+        return list(guilds)
+
+    def _do_interaction_processing(self, payload):
+        self.client.loop.create_task(self._process_interaction(payload))
 
     # Automatically register commands
     async def _auto_register_or_patch(self):
@@ -863,10 +949,10 @@ class SlashClient:
         self._global_commands = {cmd.id: cmd for cmd in commands}
 
     async def _cache_guild_commands(self):
-        for guild in self.client.guilds:
+        for guild_id in self._guilds_with_commands():
             try:
-                commands = await self.fetch_guild_commands(guild.id)
-                perms = await self.batch_fetch_guild_command_permissions(guild.id)
+                commands = await self.fetch_guild_commands(guild_id)
+                perms = await self.batch_fetch_guild_command_permissions(guild_id)
                 if len(commands) > 0:
                     # Merge commands and permissions
                     merged_commands = {}
@@ -875,24 +961,36 @@ class SlashClient:
                             cmd.permissions = perms[cmd.id]
                         merged_commands[cmd.id] = cmd
                     # Put to the dict
-                    self._guild_commands[guild.id] = merged_commands
-            except:
+                    self._guild_commands[guild_id] = merged_commands
+            except Exception:
                 pass
 
     # Component support in discord.py
     async def _send_with_components(self, channel,  content=None, *, tts=False,
-                                                    embed=None, file=None,
-                                                    components=None,
+                                                    embed=None, components=None,
+                                                    file=None, files=None,
                                                     allowed_mentions=None,
+                                                    reference=None,
                                                     **options):
+        try:
+            channel = await channel._get_channel()
+        except Exception:
+            pass
+
         state = self.client._get_state()
-        data = {**options}
+        data = {**options, "tts": tts}
 
         if content is not None:
             data["content"] = str(content)
         if embed is not None:
             data["embed"] = embed.to_dict()
-
+        if reference is not None:
+            try:
+                reference = reference.to_message_reference_dict()
+                data["message_reference"] = reference
+            except AttributeError:
+                raise discord.InvalidArgument('reference parameter must be Message or MessageReference') from None
+        
         if allowed_mentions:
             if state.allowed_mentions:
                 allowed_mentions = state.allowed_mentions.merge(allowed_mentions).to_dict()
@@ -909,9 +1007,11 @@ class SlashClient:
                 raise discord.InvalidArgument("components must be a list of ActionRow")
             data["components"] = [comp.to_dict() for comp in components]
 
+        if files is not None and len(files) > 0:
+            file = files[0]
         if file:
             try:
-                await self.client.http.request(
+                data = await self.client.http.request(
                     Route("POST", f"/channels/{channel.id}/messages"),
                     form=[
                         {
@@ -925,7 +1025,7 @@ class SlashClient:
                             "content_type": "application/octet-stream",
                         },
                     ],
-                    files=[file],
+                    files=[file]
                 )
             finally:
                 file.close()
@@ -934,19 +1034,18 @@ class SlashClient:
                 Route("POST", f"/channels/{channel.id}/messages"), json=data
             )
         return discord.Message(
-            # components=components,
             state=state,
             channel=channel,
             data=data
         )
 
     async def _edit_with_components(self, message,  content=None, *, tts=False,
-                                                    embed=None, file=None,
-                                                    components=None,
+                                                    embed=None, components=None,
+                                                    file=None, files=None,
                                                     allowed_mentions=None,
                                                     **options):
         state = self.client._get_state()
-        data = {**options}
+        data = {**options, "tts": tts}
 
         if content is not None:
             data["content"] = str(content)
@@ -969,7 +1068,9 @@ class SlashClient:
                 raise discord.InvalidArgument("components must be a list of ActionRow")
             data["components"] = [comp.to_dict() for comp in components]
         
-        if file:
+        if files is not None and len(files) > 0:
+            file = files[0]
+        if file is not None:
             try:
                 await self.client.http.request(
                     Route("PATCH", f"/channels/{message.channel.id}/messages/{message.id}"),
@@ -1011,25 +1112,31 @@ class SlashClient:
         return await asyncio.wait_for(future, timeout)
 
     # Adding relevant listeners
+    async def _on_socket_response(self, payload):
+        if payload.get("t") != "INTERACTION_CREATE":
+            return
+        self._do_interaction_processing(payload["d"])
+
     async def _on_shard_connect(self, shard_id):
-        self.client._AutoShardedClient__shards[shard_id].ws._discord_parsers['INTERACTION_CREATE'] = self._do_invokation
         self.active_shard_count += 1
-        if self.active_shard_count >= self.client.shard_count:
+        if self.active_shard_count == 1:
             await self._cache_global_commands()
-            await self._auto_register_or_patch()
             await self._cache_guild_commands()
-            self.is_ready = True
-            await self._activate_event('ready')
+            await self._auto_register_or_patch()
     
     async def _on_connect(self):
         if not isinstance(self.client, discord.AutoShardedClient):
-            self.client.ws._discord_parsers['INTERACTION_CREATE'] = self._do_invokation
             await self._cache_global_commands()
-            await self._auto_register_or_patch()
             await self._cache_guild_commands()
+            await self._auto_register_or_patch()
             self.is_ready = True
             await self._activate_event('ready')
     
+    async def _on_ready(self):
+        if isinstance(self.client, discord.AutoShardedClient):
+            self.is_ready = True
+            await self._activate_event('ready')
+
     async def _on_guild_remove(self, guild):
         if guild.id in self._guild_commands:
             del self._guild_commands[guild.id]
@@ -1069,6 +1176,8 @@ class SlashClient:
         # Don't use it
         '''
         self.client.loop.create_task(self._toggle_listeners(event_name, *args, **kwargs))
+        if event_name != "ready":
+            self.client.dispatch(event_name, *args, **kwargs)
         func = self.events.get(event_name)
         if func is not None:
             cog = get_class(func)
@@ -1077,7 +1186,7 @@ class SlashClient:
             else:
                 await func(*args, **kwargs)
 
-    async def _invoke_slash_command(self, payload):
+    async def _process_interaction(self, payload):
         '''
         # Don't use it
         '''
